"""CDM 5.4 vocabulary tables."""

from omop_cdm.dynamic.cdm54.vocabularies.vocabularies import (
    BaseConceptAncestorCdm54,
    BaseConceptCdm54,
    BaseConceptClassCdm54,
    BaseConceptRelationshipCdm54,
    BaseConceptSynonymCdm54,
    BaseDomainCdm54,
    BaseDrugStrengthCdm54,
    BaseRelationshipCdm54,
    BaseSourceToConceptMapCdm54,
    BaseSourceToConceptMapVersionCdm54,
    BaseVocabularyCdm54,
)
